class Disk:
    """A disk in the system connected to a drive sanitizer"""

    HEX_STR = "0x" # The pre-pended hex designator of a hexadecimal number

    # Table mapping each expander phy to drawer (row) and slot(column)
    # e.g. phy 14 is drawer index 0 ("A") and slot 1
    mappingTable = [[14, 15, 13, 12, 17, 16, 18, 19],
                    [22, 23, 21, 20, 25, 24, 26, 27],
                    [33, 32, 34, 35, 37, 36, 38, 39],
                    [41, 40, 42, 43, 45, 44, 46, 47]]
    DRAWERINDEXES = 4
    NUM_SLOTS = 8

    def __init__(self, address, expander, phy, linkSpeed):
        # Make sure none of input values are empty
        if (not address) or (not expander) or (not phy) or (not linkSpeed):
            raise ValueError("Not enough information to create disk")

        self.address = address
        self.expander = expander
        self.phy = phy
        self.linkSpeed = linkSpeed
        diskPosition = self.setDrawerSlot()
        self.drawer = str(diskPosition[0])
        self.slot = str(diskPosition[1])

    def setDrawerSlot(self):
        for i in range(Disk.DRAWERINDEXES):
            for j in range(Disk.NUM_SLOTS):
                currentPhy = str(Disk.mappingTable[i][j])
                if (currentPhy == self.phy):
                    drawer = translateDrawerIndex(i)
                    slot = j + 1 #Slots numbered from 1-8
                    return (drawer, slot)
        # no mapping found
        print("No map for phy found")
        return (-1, -1)


    def __repr__(self):
        driveString = self.expander + "\t\t" + self.address + "\t" + self.drawer + "\t" + self.slot + "\t" + self.phy + "\t " + str(self.linkSpeed) + " Gb/s"
        return driveString

def getDiskTableHeader():
    """
    Defines the header of the table that gets printed along with disks information. If change this function, also change
    the __repr__ function in Disk class.
    :return: A string representing the table header of disks
    """
    return "Expander Address" + "\t\t" + "SAS Address" + "\t\t" + "Drawer" + "\t" + "Slot" + "\t" + "Phy" + "\t" + " Link Speed"

def translateDrawerIndex(index):
    """
    Drawers are labels A-D, and the mappingTable is referenced by index. This functions maps the index to the corresponding
    drawer letter label
    :param index: the zero based index (rows) of mapping table
    :return: string representing the drawer label (A - D)
    """
    if (index == 0):
        return "A"
    if (index == 1):
        return "B"
    if (index == 2):
        return "C"
    if (index == 3):
        return "D"
    # anything else invalid index
    return "X"

