from Disk import *
from mapExpanders import getSanitizerList
from subprocess import Popen, PIPE

def getDiskInfo(expanderAddr, phy):
    """
    Executes g3Xutils command that displays disk information in given expander
    :param expanderAddr: the hex string address of targe expander
    :param phy: the target expander phy as a decimal string
    :return: the raw output from g3xutils command
    """
    process = Popen(["./g3Xutil", "-i", expanderAddr, "get", "phy", phy], stdout=PIPE)
    (output, err) = process.communicate()
    exit_code = process.wait();
    if (exit_code == 0):
        return output.splitlines()
    else:
        return ""


def isEntry(ENTRY_ID,  line, numArgs):
    """
    Checks if line contains entry identifier
    :param ENTRY_ID: the string identifier to be checked for
    :param line: the raw line output from g3Xutils command
    :return: true if line contains identifier
    """
    if ENTRY_ID in line:
        words = line.split()
        numWords = len(words)
        if numWords == numArgs:
            return True


def getDiskAddr(line):
    """
    Parses the line entry from g3xutil corresponding to the SAS addr of a disk
    :param line: the line output from running g3xutil
    :return: hex string of SAS addr of disk
    """
    #Address is last word in line
    words = line.split()
    addrStr = Disk.HEX_STR + words[-1]
    return addrStr

def getPhy(line):
    """
    Parses the line entry from g3xutil corresponding to the expander phy of the disk
    :param line: the line output from running g3xutil
    :return: number string of expander phy disk is connected to
    """
    #Phy is last word in line
    words = line.split()
    return words[-1]

def getLinkSpeed(line):
    """
    Parses the line entry from g3xutil corresponding to the link speed of the disk
    :param line: the line output from running g3xutil
    :return: number string of disk link speed
    """
    #Link speed is last two words of line
    words = line.split()
    linkSpeed = words[-2]
    return linkSpeed

def parseDiskRawData(rawDataLines, expanderAddr):
    """
    Parses the string input from executing g3xutils command on a disk and creates disk object
    :param rawDataLines: the output from executing g3Xutils command
    :return: a newly created disk object corresponding to the information obtained from rawDataLines input
    """
    # Constands for each entry (Line)  we are ineterested in from output of g3xutil
    SAS_ADDR_ID = "Attached SAS Address: "   # The identifier of a line containing a disk SAS Address
    SAS_ENTRY_WORDS = 4             # Number of words in sas entry
    PHY_ID = "Phy Identifier: "     # The identifier of a phy entry
    PHY_ENTRY_WORDS = 3             # NUmber of words in phy entry
    LINK_ID = "Link Speed: "        # The identifier of a link speed entry
    LINK_ENTRY_WORDS = 4            # Number of words in link speed entry
    SPEED_ID = "Gb/s"               # The identifier of Gigabit speed

    # Declare variables to hold disk information
    diskAddr = ""
    diskPhy = ""
    diskLinkSpeed = ""

    for line in rawDataLines:
        if (isEntry(SAS_ADDR_ID, line, SAS_ENTRY_WORDS)):
            diskAddr = getDiskAddr(line)

        if (isEntry(PHY_ID, line, PHY_ENTRY_WORDS)):
            diskPhy = getPhy(line)

        if isEntry(LINK_ID, line, LINK_ENTRY_WORDS) and isEntry(SPEED_ID, line, LINK_ENTRY_WORDS):
            diskLinkSpeed = getLinkSpeed(line)

    newDisk = Disk(diskAddr, expanderAddr, diskPhy, diskLinkSpeed)
    return newDisk

def getDisks(expanderAddress):
    """
    Parses output from g3xutils and generates disks objects
    :return: Array of disks objects
    """
    diskArray = []
   # Cycle through all possible phys in expander
    for drawerIndex in range(Disk.DRAWERINDEXES):
        for slotIndex in range(Disk.NUM_SLOTS):
            currentPhy = str(Disk.mappingTable[drawerIndex][slotIndex])

            rawDiskInfo = getDiskInfo(expanderAddress, currentPhy)
            try:
                newDisk = parseDiskRawData(rawDiskInfo, expanderAddress)
                # Make sure drive is linked before adding to list
                if (newDisk.linkSpeed != "-"):
                    diskArray.append(newDisk)
            except ValueError:
                print("Unable to create disk in expander: " + expanderAddress + "; Phy: " + str(phy))

    return diskArray

def main():
    """Queries the expander and outputs the connected drives information and corresponding location in sanitizer (Drawer/Slot)"""
    print("Mapping drives . . .")
    expanderList = getSanitizerList()

    print("")
    for expanderAddress in expanderList:
        print("Astek Sanitizer: " + expanderAddress)
        diskArray = getDisks(expanderAddress)
        header = getDiskTableHeader();
        print(header);
        for disk in diskArray:
            print(disk)

if __name__ == '__main__':
   main()
