from Disk import Disk
from subprocess import Popen, PIPE

def getExpandersInfo():
    """
    Executes g3Xutils command that displays expanders and their respective SAS addr
    :return: the raw output from g3xutils command
    """
    process = Popen(["./g3Xutil", "-i", "get", "avail"], stdout=PIPE)
    (output, err) = process.communicate()
    exit_code = process.wait();
    if (exit_code == 0):
        return output.splitlines()
    else:
        return ""

def isSanitizerEntry(line):
    """
    Checks if a line is the entry for an Astek Drive Sanitizer from the g3Xutils command
    (e.g. 4) A41612-ENC-01 (SAS_3X_48) (50012BE0:0007DBFF)  (0.0.0.0))
    :param line: a line string from the output of executing the g3xutils command
    :return: True if the line corresponds to the entry of an Astek Sanitizer expander
    """
    DRIVE_SAN = "A43212-SAN" #Name of the Astek Drive Saniziter
    EXPANDER_ENTRY_WORDS = 5 # Number of words in expander entry
    if DRIVE_SAN in line:
        if len(line.split()) == EXPANDER_ENTRY_WORDS:
            print(line)
            return True
    return False

def getExpanderAddr(line):
    """
    Retrieves and formats an expander address from a line reponse of g3Xutils which lists SAS Address as 4th word in line
    in-between parentheses and separated by a colon (e.g. (50012BE0:0007DBFF))
    :param line: the raw line from the output of g3xutils command (e.g. 4) A41612-ENC-01 (SAS_3X_48) (50012BE0:0007DBFF)  (0.0.0.0))
    :return: The expander address as a hex string (e.g. 0x50012BE00007DBFF)
    """
    # The SAS addr should be forth space separated string
    ADDR_INDEX = 3 # The index at which the sas address is expected to appear in the line
    words = line.split()
    rawAddr = words[ADDR_INDEX]
    # use regular expressions to manipulate string
    import re
    # Remove all non-word characters
    addrStr = re.sub('\W', '', rawAddr)
    # Add hex designator in front of address
    addrStr = Disk.HEX_STR + addrStr
    return addrStr

def getSanitizerList():
    """
    Returns a list (set) of unique drive sanitizer SAS addresses connected to the system
    :return: a set containing the hex string of SAS addresses of all Astek Drive Sanitizers connected to the system
    """
    expanderSet = set()
    # Get output from g3Xutil command with all expanders
    rawExpanderData = getExpandersInfo()

    # Iterate over all lines
    for line in rawExpanderData:
        # If line lists a drive sanitizer add SAS addr
        if (isSanitizerEntry(line)):
            expanderSet.add(getExpanderAddr(line))

    if len(expanderSet) == 0:
        print("g3xutil -i get avail")
        for line in rawExpanderData:
            print(line)
        print("Unable to find any Astek Sanitizers")
    return expanderSet
